;
; MZ-1M01 IPL-ROM TEST PROGRAM
;
;  RAM DISK for S-OS"SWORD" in 8bit mode
;

PATADR	EQU	1D19H
MPRINT	EQU	1FE2H
ETRK	EQU	20FFH
DOSMOD	EQU	2B00H
DREAD	EQU	2B07H
DWRTE	EQU	2B17H

	ORG	8000H

;
; Init PIO
;
START:	CALL	MPRINT
	DB	"Ramdisk driver replacement",0DH
	DB	"  for using 16-BIT BOARD KIT",0DH,0DH,00H

	DI
	LD	A,VECT >> 8
	LD	I,A
	IM	2
	LD	HL,IOTBL
	LD	B,8
INIL:	LD	A,(HL)
	INC	HL
	LD	C,(HL)
	INC	HL
	OUT	(C),A
	DJNZ	INIL
;
; Replace RAMDISK procedure
;
	LD	HL,PATDT
	LD	DE,RDSUB
	LD	BC,CHKPA0 - RDSUB + 1
	LDIR

	LD	HL,PATDT + (CHKPA0 - RDSUB + 1)
	LD	DE,GETPA
	LD	BC,IFLAGB - GETPA + 1
	LDIR

	LD	HL,RDSUB	; Patch for new driver
	LD	(DOSMOD + 1),HL
	LD	HL,WTSUB
	LD	(DOSMOD + 4),HL

	LD	HL,PATADR	; 1D19 LD HL,($DTAFR)
	LD	A,0C9H		;    -> RET
	LD	(HL),A

	LD	A,1Fh		; Cluster size
	LD	(ETRK),A
;
; Enter 88-IPL debug mode
;
	EI
	LD	C,1
	CALL	PUTPA

	CALL	MPRINT
	DB	"setup completed.",0DH,00H
	RET

;
; Z80A PIO initialize words
;
IOTBL:	DW	0D5H:VECT & 0FFH
	DW	0D58FH
	DW	0D587H
	DW	0D7H:(VECT & 0FFH) + 2
	DW	0D7CFH
	DW	0D77FH
	DW	0D797H
	DW	0D7FFH

PATDT:

;
; RAM DISK read/write routine
;
; HL ... memory address for read/write
; DE ... begin record number
; A  ... number of records
;

	ORG	2954H

RDSUB:	EX	AF,AF'
	CP	4
	JR	Z,RDRAM
	EX	AF,AF'
	JP	DREAD

WTSUB:	EX	AF,AF'
	CP	4
	JR	Z,WTRAM
	EX	AF,AF'
	JP	DWRTE

;
; Interrupt vector
;
VECT:	DW	INTPA
	DW	INTPB
;
; work for parameter
;
PARTBL:	DB	01H		; command
	DW	0000H		; offset
	DW	0000H		; segment
	DW	0000H		; size

RDRAM:	PUSH	IX
	PUSH	BC
	LD	IX,RDRAM2
	LD	C,01H		; read memory command
	JR	RDRAM0

WTRAM:	PUSH	IX
	PUSH	BC
	LD	IX,WTRAM1
	LD	C,02H		; write memory command

RDRAM0:	EX	AF,AF'
	PUSH	DE
	PUSH	HL

;
; check record number
;
	LD	H,0
	LD	L,A
	LD	B,A
	ADD	HL,DE
	LD	A,H
	OR	A
	JR	Z,RDRAM1
	AND	0FEH
	JR	NZ,BDREC
	LD	A,L
	CP	0F0H
	JR	NC,BDREC
RDRAM1:	CALL	SETPAR		; command
	POP	HL
	PUSH	HL
	LD	B,0
	JP	(IX)

RDRAM2:	IN	A,(0D4H)	; DUMMY
L_RDR:	CALL	GETPA
	LD	(HL),A
	INC	HL
	DJNZ	L_RDR
	DEC	E
	JR	NZ,L_RDR
	OR	A
	JR	RAMRT
BDREC:	LD	A,5		; bad record error
	SCF
RAMRT:	POP	HL
	POP	DE
	POP	BC
	POP	IX
	RET

WTRAM1:	LD	C,(HL)
	INC	HL
	CALL	PUTPA
	DJNZ	WTRAM1
	DEC	E
	JR	NZ,WTRAM1
	OR	A
	JR	RAMRT

;
; Set parameter for xfer
;
SETPAR:	LD	HL,PARTBL
	PUSH	HL
	LD	(HL),C
	INC	HL
	INC	HL
	INC	HL
	SLA	E
	RL	D
	SLA	E
	RL	D
	SLA	E
	RL	D
	SLA	E
	RL	D
	LD	(HL),E		; segment low
	INC	HL
	LD	(HL),D		; segment high
	INC	HL
	INC	HL
	LD	(HL),B		; size high

	LD	B,7
	POP	HL
SETL:	LD	C,(HL)
	INC	HL
	CALL	PUTPA
	DJNZ	SETL
	LD	E,C
	RET

;
; put 1 byte to 8088
;
PUTPA:	XOR	A
	LD	(IFLAGA),A
	LD	A,C
	OUT	(0D4H),A
CHKPA:	LD	A,(IFLAGA)
	OR	A
	JR	Z,CHKPA
CHKPA0:	RET

	ORG	2CDBH
;
; get 1 byte from 8088
;
GETPA:	LD	A,(IFLAGB)
	OR	A
	JR	Z,GETPA
	XOR	A
	LD	(IFLAGB),A
	IN	A,(0D4H)
	RET

;
; Interrupt hundler for PIO-Aport
;
INTPA:	PUSH	AF
	LD	A,1
	LD	(IFLAGA),A
	JR	INTEX
IFLAGA:	DB	0

;
; Interrupt hundler for PIO-Bport
;
INTPB:	PUSH	AF
	LD	A,1
	LD	(IFLAGB),A
INTEX:	POP	AF
	EI
	RETI
IFLAGB:	DB	0

	END	START
